"use strict";
require('reflect-metadata');
var namings = require('./namings');
var descriptions_1 = require('./descriptions');
function createHttpMethodFunction(httpMethod) {
    return function (target, key, descriptor) {
        Reflect.defineMetadata(namings.httpMethodMarker, httpMethod, descriptor.value);
    };
}
/**
 * Publish a method as an http endpoint for GET requests. If no Path is specified the path will be /.
 * The decorator is only allowed on methods.
 * @returns the decorated function
 */
function GET() { return createHttpMethodFunction(descriptions_1.HttpMethod.GET); }
exports.GET = GET;
/**
 * Publish a method as an http endpoint for POST requests. If no Path is specified the path will be /.
 * The decorator is only allowed on methods.
 * @returns the decorated function
 */
function POST() { return createHttpMethodFunction(descriptions_1.HttpMethod.POST); }
exports.POST = POST;
/**
 * Publish a method as an http endpoint for PUT requests. If no Path is specified the path will be /.
 * The decorator is only allowed on methods.
 * @returns the decorated function
 */
function PUT() { return createHttpMethodFunction(descriptions_1.HttpMethod.PUT); }
exports.PUT = PUT;
/**
 * Publish a method as an http endpoint for DELETE requests. If no Path is specified the path will be /.
 * The decorator is only allowed on methods.
 * @returns the decorated function
 */
function DELETE() { return createHttpMethodFunction(descriptions_1.HttpMethod.DELETE); }
exports.DELETE = DELETE;
/**
 * Specify the Path for the ressource. If the decorator is present at class level all methods paths will be
 * prefixed with this path.
 * @param path The path for the class or method. The path must not be start with /.
 * The slash will be added automatically.
 * @returns the decorated function
 */
function Path(path) {
    return function (target, propertyKey, descriptor) {
        if (!propertyKey && !descriptor) {
            // add meta data to the class itself - e.g. target is the constructor and propertyKey and serviceDescription are undefined
            return Reflect.defineMetadata(namings.path, path, target);
        }
        else {
            // add meta data to a function
            return Reflect.defineMetadata(namings.path, path, descriptor.value);
        }
    };
}
exports.Path = Path;
function createParamDecorator(name, pathParamKey) {
    return function (target, propertyKey, parameterIndex) {
        var existingPathParams = Reflect.getOwnMetadata(pathParamKey, target, propertyKey) || [];
        existingPathParams.push({ paramName: name, index: parameterIndex });
        Reflect.defineMetadata(pathParamKey, existingPathParams, target, propertyKey);
    };
}
/**
 * Specifies how a method parameter is evealuated. In this case the value will be taken
 * from the parameter that is specified in the path decorator.
 * @param name the name in the path that should be used to provide the parameter to the method.
 * @returns the decorated function
 */
function PathParam(name) { return createParamDecorator(name, namings.pathParam); }
exports.PathParam = PathParam;
/**
 * Specifies how a method parameter is evaluated. In this case the value will be taken
 * from a http header.
 * @param name the name in the http header that should be used to provide the parameter to the method.
 * @returns the decorated function
 */
function HeaderParam(name) { return createParamDecorator(name, namings.headerParam); }
exports.HeaderParam = HeaderParam;
/**
 * Specifies how a method parameter is evaluated. In this case the value will be taken
 * from a url query parameter.
 * @param name the name of the query param that should be used to provide the parameter to the method.
 * @returns the decorated function
 */
function QueryParam(name) { return createParamDecorator(name, namings.queryParam); }
exports.QueryParam = QueryParam;
/**
 * Specifies how a method parameter is evaluated. In this case the value will be taken
 * from the context (for example the current Request).
 * @param contextType the ContextTypes that should be used to provide the parameter to the method.
 * @returns the decorated function
 */
function Context(contextType) { return createParamDecorator(descriptions_1.ContextTypes[contextType], namings.contextParam); }
exports.Context = Context;
//# sourceMappingURL=decorators.js.map